package com.nisum.offertransactionservice.service;

import com.nisum.offertransactionservice.converter.OfferConverter;
import com.nisum.offertransactionservice.converter.PEResponseToOfferTransactionResConverter;
import com.nisum.offertransactionservice.dao.OfferLookupRepo;
import com.nisum.offertransactionservice.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientResponseException;
import org.springframework.web.client.RestTemplate;

import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

@Service
@PropertySource("classpath:application.properties")
public class OfferCallingPEService {
    @Autowired
    private OfferConverter offerConverter;

    @Autowired
    private OfferLookupRepo offerLookupRepo;

    @Autowired
    private PEResponseToOfferTransactionResConverter peResponseToOfferTransactionResConverter;

    @Autowired
    private RestTemplate restTemplate;

    @Value("${promotion.engine.calculate.discount.url}")
    private String promotionEngineUrl ;

    public OfferTransactionResponse getDiscountedItemList(OfferTransactionRequest offerTransactionRequest) {
        List<OfferLookup> eligibleOffer = newArrayList();
        offerLookupRepo.findAll().forEach(eligibleOffer::add);
        OfferTransactionResponse offerTransactionResponse =null;
        //TODO validate whether we got the offers from DB or not
        //TODO DB call success or not
        PERequest peRequest = offerConverter.apply(offerTransactionRequest);
        peRequest.setEligibleOffers(eligibleOffer);
        ResponseEntity<Object> peResponse = getPeResponseResponseEntity(peRequest);
        if(peResponse.getBody() instanceof  PEResponse){
            offerTransactionResponse = peResponseToOfferTransactionResConverter.apply((PEResponse)peResponse.getBody());
        }


        return offerTransactionResponse;
    }

    private ResponseEntity<Object> getPeResponseResponseEntity(PERequest peRequest) {
        try{
            restTemplate.postForEntity("http://localhost:8081"+promotionEngineUrl,peRequest,PEResponse.class);
        }catch(RestClientResponseException e){
            return ResponseEntity
                    .status(e.getRawStatusCode())
                    .body(e.getResponseBodyAsString());
        }

        return null;
    }
}
